@@ -1,5 +1,7 @@
 //! Tactics for Logical NOT.
 
+#![allow(unreachable_code)]
+
 use crate::*;
 
 /// `a => ¬¬a`.
@@ -16,3 +18,8 @@ pub fn rev_double<A: DProp>(f: Not<Not<A>>) -> A {
         Right(not_a) => match f(not_a) {},
     }
 }
+
+/// `¬a ⋀ a => b`.
+pub fn absurd<A: Prop, B: Prop>(f: Not<A>, g: A) -> B {
+    imply::absurd()(f(g))
+}
